package fr.gouv.beta.selexpert.dto;

import fr.gouv.beta.selexpert.model.Role;
import fr.gouv.beta.selexpert.model.User;
import fr.gouv.beta.selexpert.model.UserClerk;
import fr.gouv.beta.selexpert.model.UserMagistrate;
import java.time.LocalDateTime;
import lombok.Builder;
import lombok.Data;

@Data
@Builder
public class CourtUser {
  private Integer id;
  private String name;
  private String email;
  private Role role;
  private String appealCourt;
  private String jurisdiction;
  private CourtUserServiceDto service;
  private LocalDateTime activatedOn;
  private int requestCount;

  public static CourtUser from(User user) {
    if (!user.getRole().isCourt()) {
      return null;
    }
    var jurisdiction =
        Role.MAGISTRATE.equals(user.getRole())
            ? ((UserMagistrate) user).getJurisdiction()
            : ((UserClerk) user).getJurisdiction();
    var service =
        CourtUserServiceDto.from(
            Role.MAGISTRATE.equals(user.getRole())
                ? ((UserMagistrate) user).getService()
                : ((UserClerk) user).getService());
    return CourtUser.builder()
        .name(user.getName())
        .email(user.getEmail())
        .appealCourt(jurisdiction.getAppealCourt().getName())
        .jurisdiction(jurisdiction.getName())
        .service(service)
        .role(user.getRole())
        .activatedOn(user.getActivatedOn())
        .build();
  }

  public static CourtUser fromLight(User user) {
    if (!user.getRole().isCourt()) {
      return null;
    }
    var jurisdiction =
        Role.MAGISTRATE.equals(user.getRole())
            ? ((UserMagistrate) user).getJurisdiction()
            : ((UserClerk) user).getJurisdiction();
    return CourtUser.builder()
        .id(user.getId())
        .name(user.getName())
        .jurisdiction(jurisdiction.getName())
        .role(user.getRole())
        .build();
  }
}
