package fr.gouv.beta.selexpert.dto;

import jakarta.validation.constraints.AssertTrue;
import jakarta.validation.constraints.NotNull;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Objects;
import lombok.Data;

@Data
public class ExpertAvailabilityForm {

  @NotNull private Boolean available;
  private LocalDate unavailableFrom;
  private LocalDate unavailableUntil;

  @AssertTrue
  private boolean isDatesMakeSense() {
    if (available || (Objects.isNull(unavailableFrom) && Objects.isNull(unavailableUntil)))
      return true;
    var now = LocalDate.now(ZoneId.of("Europe/Paris"));
    if (Objects.nonNull(unavailableFrom) && unavailableFrom.isBefore(now)) return false;
    if (Objects.nonNull(unavailableUntil) && !unavailableUntil.isAfter(now)) return false;
    if (Objects.nonNull(unavailableFrom) && Objects.nonNull(unavailableUntil)) {
      return !unavailableUntil.isBefore(unavailableFrom);
    }
    return true;
  }
}
