package fr.gouv.beta.selexpert.dto;

import fr.gouv.beta.selexpert.model.Expert;
import java.time.LocalDate;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;
import lombok.Builder;
import lombok.Data;

@Data
@Builder
public class ExpertDto {
  private Integer id;
  private String appealCourt;
  private String title;
  private String name;
  private String email;
  private String phoneMobile;
  private String phoneLandline;
  private List<ExpertExpertiseDto> expertises;
  private LocalDate unavailableFrom;
  private LocalDate unavailableUntil;
  private boolean activated;
  private String address;
  private String zipCode;
  private String city;
  private String specialty;
  private boolean deleted;
  private boolean manuallyAdded;
  private boolean honorary;
  private boolean notRegistered;
  private int acceptedRequestCount;
  private String geoConstraint;
  private Boolean opalexe;

  public static ExpertDto from(Expert expert) {
    var expertDto = ExpertDto.builder();
    expertDto.id(expert.getId());
    expertDto.appealCourt(expert.getAppealCourt().getName());
    expertDto.title(expert.getTitle());
    expertDto.name(expert.getName());
    expertDto.email(expert.getEmail());
    expertDto.phoneMobile(expert.getPhoneMobile());
    expertDto.phoneLandline(expert.getPhoneLandline());
    expertDto.expertises(
        expert.getExpertExpertisesNew().stream()
            .sorted(Comparator.comparing(expertise -> expertise.getExpertise().getId()))
            .map(
                expertise ->
                    ExpertExpertiseDto.builder()
                        .code(expertise.getExpertise().getCode())
                        .label(expertise.getExpertise().getLabel())
                        .specialty(expertise.getSpecialty())
                        .fromAppealCourt(expertise.isFromAppealCourt())
                        .fromCassationCourt(expertise.isFromCassationCourt())
                        .build())
            .collect(Collectors.toList()));
    expertDto.specialty(expert.getSpecialty());
    expertDto.address(expert.getAddress());
    expertDto.zipCode(expert.getZipCode());
    expertDto.city(expert.getCity());
    expertDto.deleted(Optional.ofNullable(expert.getDeleted()).orElse(false));
    expertDto.manuallyAdded(Optional.ofNullable(expert.getManuallyAdded()).orElse(false));
    expertDto.honorary(Optional.ofNullable(expert.getHonorary()).orElse(false));
    expertDto.notRegistered(Optional.ofNullable(expert.getNotRegistered()).orElse(false));
    if (Objects.nonNull(expert.getUserExpert())) {
      expertDto.unavailableFrom(expert.getUserExpert().getUnavailableFrom());
      expertDto.unavailableUntil(expert.getUserExpert().getUnavailableUntil());
      expertDto.activated(true);
      expertDto.geoConstraint(expert.getUserExpert().getGeoConstraint());
      expertDto.opalexe(expert.getUserExpert().getOpalexe());
    }
    return expertDto.build();
  }
}
