package fr.gouv.beta.selexpert.dto;

import fr.gouv.beta.selexpert.model.NoteType;
import fr.gouv.beta.selexpert.model.Request;
import fr.gouv.beta.selexpert.model.RequestExpertStatus;
import fr.gouv.beta.selexpert.model.RequestStatus;
import fr.gouv.beta.selexpert.util.DateUtils;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import lombok.Builder;
import lombok.Data;

@Builder
@Data
public class RequestDto {
  private Integer id;
  private LocalDateTime createdOn;
  private String procedureId;
  private String extraInfo;
  private boolean isUrgent;
  private MagistrateDto magistrate;
  private String clerk;
  private String jurisdiction;
  private List<RequestExpertDto> requestExperts;
  private List<ClerkDto> clerks;
  private RequestStatus status;
  private List<FileDto> files;
  private List<NoteDto> internalNotes;
  private List<NoteDto> questions;
  private boolean isArchived;
  private LocalDate deliberationDate;

  public static RequestDto from(Request request, Integer weekdaysToReply) {
    return RequestDto.builder()
        .id(request.getId())
        .createdOn(request.getCreatedOn())
        .procedureId(request.getProcedureId())
        .extraInfo(request.getExtraInfo())
        .isUrgent(request.isUrgent())
        .status(request.getStatus())
        .jurisdiction(request.getJurisdiction().getName())
        .magistrate(MagistrateDto.from(request.getMagistrate()))
        .clerk(Objects.nonNull(request.getClerk()) ? request.getClerk().getName() : null)
        .requestExperts(
            request.getRequestExperts().stream()
                .map(
                    requestExpert ->
                        new RequestExpertDto(
                            ExpertDto.from(requestExpert.getExpert()),
                            requestExpert.getStatus(),
                            requestExpert.getUpdatedOn(),
                            requestExpert.getRefusalReason(),
                            requestExpert.getRefusalJustification(),
                            List.of(RequestExpertStatus.SENDING, RequestExpertStatus.SENT)
                                    .contains(requestExpert.getStatus())
                                ? DateUtils.getDeadline(
                                        requestExpert.getUpdatedOn(), weekdaysToReply)
                                    .toLocalDate()
                                : null))
                .toList())
        .clerks(
            request.getRequestClerks().stream()
                .map(requestClerk -> ClerkDto.from(requestClerk, true))
                .sorted(Comparator.comparing(ClerkDto::getName))
                .collect(Collectors.toList()))
        .files(request.getFiles().stream().map(FileDto::from).collect(Collectors.toList()))
        .internalNotes(
            request.getNotes().stream()
                .filter(note -> note.getType() == NoteType.INTERNAL)
                .map(NoteDto::from)
                .collect(Collectors.toList()))
        .questions(
            request.getNotes().stream()
                .filter(note -> note.getType() == NoteType.QUESTION)
                .map(
                    q ->
                        NoteDto.from(
                            q,
                            request.getNotes().stream()
                                .filter(
                                    a ->
                                        a.getType() == NoteType.ANSWER
                                            && a.getQuestionId().equals(q.getId()))
                                .collect(Collectors.toList())))
                .collect(Collectors.toList()))
        .isArchived(request.isArchived())
        .deliberationDate(request.getDeliberationDate())
        .build();
  }
}
