package fr.gouv.beta.selexpert.dto;

import fr.gouv.beta.selexpert.model.FileType;
import fr.gouv.beta.selexpert.model.NoteType;
import fr.gouv.beta.selexpert.model.RefusalReason;
import fr.gouv.beta.selexpert.model.RequestExpert;
import fr.gouv.beta.selexpert.model.RequestExpertStatus;
import fr.gouv.beta.selexpert.model.User;
import fr.gouv.beta.selexpert.util.DateUtils;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;
import lombok.Builder;
import lombok.Data;

@Builder
@Data
public class RequestDtoForExpert {
  private Integer id;
  private String procedureId;
  private String extraInfo;
  private LocalDateTime sentOn;
  private LocalDate deadline;
  private RequestExpertStatus status;
  private LocalDateTime updatedOn;
  private RefusalReason refusalReason;
  private String refusalJustification;
  private String jurisdiction;
  private List<ClerkDto> clerks;
  private MagistrateDto magistrate;
  private List<FileDto> files;
  private List<NoteDto> questions;
  private boolean archived;

  public static RequestDtoForExpert from(RequestExpert requestExpert, Integer weekdaysToReply) {
    var request = requestExpert.getRequest();
    var requestInfoForExpert =
        RequestDtoForExpert.builder()
            .id(request.getId())
            .procedureId(request.getProcedureId())
            .extraInfo(request.getExtraInfo())
            .sentOn(requestExpert.getSentOn())
            .deadline(
                Objects.nonNull(requestExpert.getSentOn())
                    ? DateUtils.getDeadline(requestExpert.getSentOn(), weekdaysToReply)
                        .toLocalDate()
                    : null)
            .status(requestExpert.getStatus())
            .updatedOn(requestExpert.getUpdatedOn())
            .archived(request.isArchived())
            .jurisdiction(request.getJurisdiction().getName())
            .clerks(
                request.getRequestClerks().stream()
                    .map(requestClerk -> ClerkDto.from(requestClerk, false))
                    .collect(Collectors.toList()))
            .magistrate(MagistrateDto.from(request.getMagistrate()))
            .files(
                request.getFiles().stream()
                    .filter(
                        file ->
                            FileType.ATTACHMENT.equals(file.getType())
                                || (FileType.DECISION.equals(file.getType())
                                    && RequestExpertStatus.ACCEPTED.equals(
                                        requestExpert.getStatus())))
                    .map(FileDto::from)
                    .collect(Collectors.toList()))
            .questions(
                request.getNotes().stream()
                    .filter(
                        note ->
                            note.getType() == NoteType.QUESTION
                                && note.getUser()
                                    .getId()
                                    .equals(
                                        Optional.ofNullable(
                                                requestExpert.getExpert().getUserExpert())
                                            .map(User::getId)
                                            .orElse(-1)))
                    .map(
                        q ->
                            NoteDto.from(
                                q,
                                request.getNotes().stream()
                                    .filter(
                                        a ->
                                            a.getType() == NoteType.ANSWER
                                                && a.getQuestionId().equals(q.getId()))
                                    .collect(Collectors.toList())))
                    .collect(Collectors.toList()));
    if (RequestExpertStatus.REFUSED.equals(requestExpert.getStatus())) {
      requestInfoForExpert.refusalReason(requestExpert.getRefusalReason());
      requestInfoForExpert.refusalJustification(requestExpert.getRefusalJustification());
    }
    return requestInfoForExpert.build();
  }
}
