package fr.gouv.beta.selexpert.dto;

import fr.gouv.beta.selexpert.model.RefusalReason;
import jakarta.validation.constraints.AssertTrue;
import jakarta.validation.constraints.NotNull;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Objects;
import lombok.Builder;
import lombok.Data;

@Data
@Builder
public class RequestRefusalForm {
  @NotNull private RefusalReason reason;
  private String justification;
  private LocalDate unavailableUntil;

  @AssertTrue
  private boolean isJustificationOk() {
    if (RefusalReason.OTHER.equals(reason)) {
      if (Objects.isNull(justification)) return false;
      justification = justification.trim();
      return justification.length() >= 20 && justification.length() <= 1000;
    }
    return Objects.isNull(justification);
  }

  @AssertTrue
  private boolean isDatesMakeSense() {
    return Objects.isNull(unavailableUntil)
        || unavailableUntil.isAfter(LocalDate.now(ZoneId.of("Europe/Paris")));
  }
}
