package fr.gouv.beta.selexpert.dto;

import fr.gouv.beta.selexpert.model.AppealCourt;
import fr.gouv.beta.selexpert.model.Expert;
import fr.gouv.beta.selexpert.model.Jurisdiction;
import fr.gouv.beta.selexpert.model.Role;
import fr.gouv.beta.selexpert.model.User;
import fr.gouv.beta.selexpert.model.UserClerk;
import fr.gouv.beta.selexpert.model.UserExpert;
import fr.gouv.beta.selexpert.model.UserMagistrate;
import lombok.*;

@NoArgsConstructor
@AllArgsConstructor
@Builder
@Data
public class UserInfo {
  private Integer id;
  private String name;
  private Role role;
  private String accessToken;
  private ExpertDto expert;
  private AppealCourt appealCourt;
  private Jurisdiction jurisdiction;
  private CourtUserServiceDto service;
  private String phoneMobile;
  private String phoneLandline;
  private boolean allNotifications;

  public static UserInfo from(User user) {
    var userInfo =
        UserInfo.builder()
            .id(user.getId())
            .name(user.getName())
            .role(user.getRole())
            .allNotifications(!Boolean.FALSE.equals(user.getAllNotifications()));
    switch (user.getRole()) {
      case EXPERT -> {
        Expert expert = ((UserExpert) user).getExpert();
        userInfo.expert(ExpertDto.from((expert)));
        userInfo.appealCourt(expert.getAppealCourt());
      }
      case MAGISTRATE -> {
        UserMagistrate userMagistrate = (UserMagistrate) user;
        userInfo.jurisdiction(userMagistrate.getJurisdiction());
        userInfo.service(CourtUserServiceDto.from(userMagistrate.getService()));
      }
      case CLERK -> {
        UserClerk userClerk = (UserClerk) user;
        userInfo.jurisdiction(userClerk.getJurisdiction());
        userInfo.service(CourtUserServiceDto.from(userClerk.getService()));
        userInfo.phoneMobile(userClerk.getPhoneMobile());
        userInfo.phoneLandline(userClerk.getPhoneLandline());
      }
    }
    return userInfo.build();
  }
}
