package fr.gouv.beta.selexpert.listener;

import fr.gouv.beta.selexpert.model.EnversRevision;
import fr.gouv.beta.selexpert.model.Role;
import fr.gouv.beta.selexpert.repository.UserRepository;
import java.util.Objects;
import lombok.extern.slf4j.Slf4j;
import org.hibernate.envers.RevisionListener;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

@Slf4j
public class EnversRevisionListener implements RevisionListener {

  private final UserRepository userRepository;

  public EnversRevisionListener(@Lazy UserRepository userRepository) {
    this.userRepository = userRepository;
  }

  @Override
  public void newRevision(Object revisionEntity) {
    EnversRevision rev = (EnversRevision) revisionEntity;
    try {
      var id = SecurityContextHolder.getContext().getAuthentication().getName();
      var user = userRepository.findById(Integer.valueOf(id)).orElseThrow();
      rev.setUsername(
          (user.getRole() == Role.ADMIN && isExpertImport()) ? "import" : user.getName());
    } catch (Exception e) {
      log.error("can't get username");
      rev.setUsername("???");
    }
  }

  private boolean isExpertImport() {
    return Objects.nonNull(RequestContextHolder.getRequestAttributes())
        && ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes())
            .getRequest()
            .getRequestURI()
            .contains("/admin/import/");
  }
}
