package fr.gouv.beta.selexpert.model;

import static org.hibernate.envers.RelationTargetAuditMode.NOT_AUDITED;

import jakarta.persistence.*;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.ToString;
import org.hibernate.envers.Audited;

@Entity
@Table(name = "court_user_court_service", schema = "public")
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@Audited(withModifiedFlag = true)
public class CourtUserCourtService {

  @EmbeddedId private CourtUserCourtServiceId id;

  @ManyToOne
  @MapsId("courtUserId")
  @ToString.Exclude
  @EqualsAndHashCode.Exclude
  @Audited(targetAuditMode = NOT_AUDITED)
  private User courtUser;

  @ManyToOne
  @MapsId("courtServiceId")
  @ToString.Exclude
  @EqualsAndHashCode.Exclude
  @Audited(targetAuditMode = NOT_AUDITED)
  private CourtService courtService;

  @Column @EqualsAndHashCode.Exclude private String other;

  public CourtUserCourtService(User courtUser, CourtService courtService, String other) {
    this.id = new CourtUserCourtServiceId(courtUser.getId(), courtService.getId());
    this.courtUser = courtUser;
    this.courtService = courtService;
    this.other = courtService.isFreeInput() ? other : null;
  }
}
