package fr.gouv.beta.selexpert.model;

import jakarta.persistence.*;
import java.util.List;
import java.util.Set;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.annotations.ColumnDefault;
import org.hibernate.envers.Audited;

@Entity
@Table(name = "expert", schema = "public")
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class Expert {

  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  private Integer id;

  @ManyToOne(optional = false)
  private AppealCourt appealCourt;

  @Column(name = "title")
  private String title;

  @Column(name = "name", nullable = false)
  @ColumnDefault("''")
  @Audited(withModifiedFlag = true)
  private String name;

  @Column(name = "email", unique = true)
  @Audited(withModifiedFlag = true)
  private String email;

  @Column(name = "phone_mobile")
  @Audited(withModifiedFlag = true)
  private String phoneMobile;

  @Column(name = "phone_landline")
  @Audited(withModifiedFlag = true)
  private String phoneLandline;

  @OneToMany(mappedBy = "expert", cascade = CascadeType.ALL, orphanRemoval = true)
  private Set<ExpertExpertise> expertExpertises;

  @OneToMany(mappedBy = "expert", cascade = CascadeType.ALL, orphanRemoval = true)
  private Set<ExpertExpertiseNew> expertExpertisesNew;

  @OneToOne(mappedBy = "expert")
  private UserExpert userExpert;

  @OneToOne private ExpertNew expertNew;

  @Column(name = "address")
  @Audited(withModifiedFlag = true)
  private String address;

  @Column(name = "zip_code")
  @Audited(withModifiedFlag = true)
  private String zipCode;

  @Column(name = "city")
  @Audited(withModifiedFlag = true)
  private String city;

  @Column(name = "specialty", columnDefinition = "TEXT", length = 2000)
  @Audited(withModifiedFlag = true)
  private String specialty;

  @Column(name = "deleted")
  @Audited(withModifiedFlag = true)
  private Boolean deleted;

  @Column(name = "manually_added")
  private Boolean manuallyAdded;

  @Column(name = "honorary")
  private Boolean honorary;

  @Column(name = "not_registered")
  private Boolean notRegistered;

  @OneToMany(mappedBy = "expert")
  private List<RequestExpert> requestExperts;
}
