package fr.gouv.beta.selexpert.model;

import jakarta.persistence.*;
import java.util.Set;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.annotations.ColumnDefault;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

@Entity
@Table(name = "expert_new", schema = "public")
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
@Audited(withModifiedFlag = true)
public class ExpertNew {

  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  private Integer id;

  @ManyToOne(optional = false)
  @NotAudited
  private AppealCourt appealCourt;

  @Column(name = "esabora_id")
  private String esaboraId;

  @Column(name = "type")
  @Enumerated(EnumType.STRING)
  private ExpertType type;

  @Column(name = "name", nullable = false)
  private String name;

  @Column(name = "first_name", nullable = false)
  @ColumnDefault("''")
  private String firstName;

  @Column(name = "address", columnDefinition = "TEXT", length = 1000)
  private String address;

  @Column(name = "zip_code")
  private String zipCode;

  @Column(name = "city")
  private String city;

  @Column(name = "email")
  private String email;

  @Column(name = "phone_mobile")
  private String phoneMobile;

  @Column(name = "phone_landline")
  private String phoneLandline;

  @Column(name = "phone_fax")
  private String phoneFax;

  @OneToMany(
      mappedBy = "expert",
      cascade = CascadeType.ALL,
      orphanRemoval = true,
      fetch = FetchType.EAGER)
  private Set<ExpertNewExpertiseNew> expertExpertises;

  // internal

  @Column(name = "_deleted", nullable = false)
  private boolean deleted;

  @Transient private boolean application2024;
}
