package fr.gouv.beta.selexpert.model;

import fr.gouv.beta.selexpert.AppException;
import jakarta.persistence.*;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import lombok.Data;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.RequiredArgsConstructor;
import org.apache.commons.collections4.IterableUtils;
import org.hibernate.envers.Audited;

@Entity
@Table(name = "expert_raw_data", schema = "public")
@Data
@NoArgsConstructor
@Audited(withModifiedFlag = true)
public class ExpertRawData {

  @Id
  @Column(name = "id_expert_categorie", nullable = false, unique = true)
  private String idExpertCategorie;

  @Column(name = "id_ca", nullable = false)
  private String idCa;

  @Column(name = "ca", nullable = false)
  private String ca;

  @Column(name = "civilite", nullable = false)
  private String civilite;

  @Column(name = "nom", nullable = false)
  private String nom;

  @Column(name = "nom_usage", nullable = false)
  private String nomUsage;

  @Column(name = "prenom", nullable = false)
  private String prenom;

  @Column(name = "categorie", nullable = false)
  private String categorie;

  @Column(name = "rubrique", nullable = false)
  private String rubrique;

  @Column(name = "specialite", nullable = false)
  private String specialite;

  @Column(name = "sous-specialite", nullable = false)
  private String sousSpecialite;

  @Column(name = "periode_probatoire", nullable = false)
  private String periodeProbatoire;

  @Column(name = "date_premiere_inscription", nullable = false)
  private String datePremiereInscription;

  @Column(name = "date_reinscription", nullable = false)
  private String dateReinscription;

  @Column(name = "date_debut_periode_probatoire", nullable = false)
  private String dateDebutPeriodeProbatoire;

  @Column(name = "date_fin_periode_probatoire", nullable = false)
  private String dateFinPeriodeProbatoire;

  @Column(name = "adresse1", nullable = false)
  private String adresse1;

  @Column(name = "adresse2", nullable = false)
  private String adresse2;

  @Column(name = "adresse3", nullable = false)
  private String adresse3;

  @Column(name = "lieu_dit", nullable = false)
  private String lieuDit;

  @Column(name = "ville", nullable = false)
  private String ville;

  @Column(name = "code_postal", nullable = false)
  private String codePostal;

  @Column(name = "departement", nullable = false)
  private String departement;

  @Column(name = "courriel", nullable = false)
  private String courriel;

  @Column(name = "fax", nullable = false)
  private String fax;

  @Column(name = "mobile", nullable = false)
  private String mobile;

  @Column(name = "fixe", nullable = false)
  private String fixe;

  @Column(name = "date_application", nullable = false)
  private String dateApplication;

  @Column(name = "date_fermeture", nullable = false)
  private String dateFermeture;

  @Column(name = "id_expert", nullable = false)
  private String idExpert;

  public static ExpertRawData fromRow(String[] row) {
    ExpertRawData expertRawData = new ExpertRawData();
    expertRawData.setIdExpertCategorie(row[FileHeader.pos(FileHeader.ID_EXPERT_CATEGORIE)]);
    expertRawData.setIdCa(row[FileHeader.pos(FileHeader.ID_CA)]);
    expertRawData.setCa(row[FileHeader.pos(FileHeader.CA)]);
    expertRawData.setCivilite(row[FileHeader.pos(FileHeader.CIVILITE)]);
    expertRawData.setNom(row[FileHeader.pos(FileHeader.NOM)]);
    expertRawData.setNomUsage(row[FileHeader.pos(FileHeader.NOM_USAGE)]);
    expertRawData.setPrenom(row[FileHeader.pos(FileHeader.PRENOM)]);
    expertRawData.setCategorie(row[FileHeader.pos(FileHeader.CATEGORIE)]);
    expertRawData.setRubrique(row[FileHeader.pos(FileHeader.RUBRIQUE)]);
    expertRawData.setSpecialite(row[FileHeader.pos(FileHeader.SPECIALITE)]);
    expertRawData.setSousSpecialite(row[FileHeader.pos(FileHeader.SOUS_SPECIALITE)]);
    expertRawData.setPeriodeProbatoire(row[FileHeader.pos(FileHeader.PERIODE_PROBATOIRE)]);
    expertRawData.setDatePremiereInscription(
        row[FileHeader.pos(FileHeader.DATE_PREMIERE_INSCRIPTION)]);
    expertRawData.setDateReinscription(row[FileHeader.pos(FileHeader.DATE_REINSCRIPTION)]);
    expertRawData.setDateDebutPeriodeProbatoire(
        row[FileHeader.pos(FileHeader.DATE_DEBUT_PERIODE_PROBATOIRE)]);
    expertRawData.setDateFinPeriodeProbatoire(
        row[FileHeader.pos(FileHeader.DATE_FIN_PERIODE_PROBATOIRE)]);
    expertRawData.setAdresse1(row[FileHeader.pos(FileHeader.ADRESSE1)]);
    expertRawData.setAdresse2(row[FileHeader.pos(FileHeader.ADRESSE2)]);
    expertRawData.setAdresse3(row[FileHeader.pos(FileHeader.ADRESSE3)]);
    expertRawData.setLieuDit(row[FileHeader.pos(FileHeader.LIEU_DIT)]);
    expertRawData.setVille(row[FileHeader.pos(FileHeader.VILLE)]);
    expertRawData.setCodePostal(row[FileHeader.pos(FileHeader.CODE_POSTAL)]);
    expertRawData.setDepartement(row[FileHeader.pos(FileHeader.DEPARTEMENT)]);
    expertRawData.setCourriel(row[FileHeader.pos(FileHeader.COURRIEL)]);
    expertRawData.setFax(row[FileHeader.pos(FileHeader.FAX)]);
    expertRawData.setMobile(row[FileHeader.pos(FileHeader.MOBILE)]);
    expertRawData.setFixe(row[FileHeader.pos(FileHeader.FIXE)]);
    expertRawData.setDateApplication(row[FileHeader.pos(FileHeader.DATE_APPLICATION)]);
    expertRawData.setDateFermeture(row[FileHeader.pos(FileHeader.DATE_FERMETURE)]);
    expertRawData.setIdExpert(row[FileHeader.pos(FileHeader.ID_EXPERT)]);
    return expertRawData;
  }

  public boolean isContainedIn(List<ExpertRawData> expertRawDataList) {
    ExpertRawData found =
        IterableUtils.find(
            expertRawDataList, e -> e.getIdExpertCategorie().equals(idExpertCategorie));
    if (Objects.nonNull(found) && !this.getIdExpert().equals(found.getIdExpert())) {
      // extra check: assuming that id_expert_categorie is always linked to the same id_expert
      throw new AppException("idExpert changed for idExpertCategorie " + idExpertCategorie);
    }
    return Objects.nonNull(found);
  }

  @Getter
  @RequiredArgsConstructor
  public enum FileHeader {
    ID_EXPERT_CATEGORIE("id_expert_categorie"),
    ID_CA("id_ca"),
    CA("ca"),
    CIVILITE("civilite"),
    NOM("nom"),
    NOM_USAGE("nom_usage"),
    PRENOM("prenom"),
    CATEGORIE("categorie"),
    RUBRIQUE("rubrique"),
    SPECIALITE("specialite"),
    SOUS_SPECIALITE("sous-specialite"),
    PERIODE_PROBATOIRE("periode_probatoire"),
    DATE_PREMIERE_INSCRIPTION("date_premiere_inscription"),
    DATE_REINSCRIPTION("date_reinscription"),
    DATE_DEBUT_PERIODE_PROBATOIRE("date_debut_periode_probatoire"),
    DATE_FIN_PERIODE_PROBATOIRE("date_fin_periode_probatoire"),
    ADRESSE1("adresse1"),
    ADRESSE2("adresse2"),
    ADRESSE3("adresse3"),
    LIEU_DIT("lieu_dit"),
    VILLE("ville"),
    CODE_POSTAL("code_postal"),
    DEPARTEMENT("departement"),
    COURRIEL("courriel"),
    FAX("fax"),
    MOBILE("mobile"),
    FIXE("fixe"),
    DATE_APPLICATION("date_application"),
    DATE_FERMETURE("date_fermeture"),
    ID_EXPERT("id_expert");

    private final String label;

    public static List<String> headers() {
      return Arrays.stream(FileHeader.values())
          .map(FileHeader::getLabel)
          .collect(Collectors.toList());
    }

    public static int pos(FileHeader header) {
      return IntStream.range(0, FileHeader.values().length)
          .filter(i -> FileHeader.values()[i].equals(header))
          .findFirst()
          .orElseThrow();
    }
  }
}
