package fr.gouv.beta.selexpert.model;

import jakarta.persistence.*;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

@Entity
@Table(name = "request", schema = "public")
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class Request {

  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  private Integer id;

  @ManyToOne(optional = true) // TODO set to false
  private Jurisdiction jurisdiction;

  @ManyToOne(optional = false)
  private UserMagistrate magistrate;

  @ManyToOne private UserClerk clerk; // if clerk created on behalf of magistrate

  @OneToMany(
      mappedBy = "request",
      cascade = CascadeType.ALL,
      orphanRemoval = true,
      fetch = FetchType.EAGER)
  @OrderBy(value = "ranking ASC")
  @Builder.Default
  private List<RequestExpert> requestExperts = new ArrayList<>();

  @ManyToMany(fetch = FetchType.EAGER)
  @JoinTable(
      name = "request_clerk",
      joinColumns = @JoinColumn(name = "request_id"),
      inverseJoinColumns = @JoinColumn(name = "clerk_id"))
  @Builder.Default
  private List<UserClerk> requestClerks = new ArrayList<>();

  @Column(name = "created_on", nullable = false)
  private LocalDateTime createdOn;

  @Column(name = "procedure_id", nullable = false)
  private String procedureId;

  @Column(name = "extra_info", columnDefinition = "TEXT", length = 1000)
  //  @Audited(withModifiedFlag = true)
  private String extraInfo;

  @Column(name = "is_urgent", nullable = false)
  private boolean urgent;

  @Column(name = "status", nullable = false)
  @Enumerated(EnumType.STRING)
  //  @Audited(withModifiedFlag = true)
  private RequestStatus status;

  @OneToMany(
      mappedBy = "request",
      cascade = CascadeType.ALL,
      orphanRemoval = true,
      fetch = FetchType.EAGER)
  @OrderBy(value = "ranking ASC")
  @Builder.Default
  private List<File> files = new ArrayList<>();

  @OneToMany(
      mappedBy = "request",
      cascade = CascadeType.ALL,
      orphanRemoval = true,
      fetch = FetchType.EAGER)
  @OrderBy(value = "createdOn ASC")
  @Builder.Default
  private List<Note> notes = new ArrayList<>();

  @Column(name = "is_archived", nullable = false, columnDefinition = "boolean default false")
  private boolean archived;

  @Column private LocalDate deliberationDate;
}
