package fr.gouv.beta.selexpert.model;

import jakarta.persistence.*;
import java.time.LocalDateTime;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.ToString;

@Entity
@Table(name = "request_expert", schema = "public")
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode
public class RequestExpert {

  @EmbeddedId private RequestExpertId id;

  @ManyToOne
  @MapsId("requestId")
  @ToString.Exclude
  @EqualsAndHashCode.Exclude
  private Request request;

  @ManyToOne
  @MapsId("expertId")
  private Expert expert;

  @Column(nullable = false)
  private Integer ranking;

  @Column(nullable = false)
  @Enumerated(EnumType.STRING)
  private RequestExpertStatus status;

  @Column private LocalDateTime sentOn;

  @Column(nullable = false)
  private LocalDateTime updatedOn;

  @Column
  @Enumerated(EnumType.STRING)
  private RefusalReason refusalReason;

  @Column(columnDefinition = "TEXT", length = 1000)
  private String refusalJustification;

  public RequestExpert(Request request, Expert expert, Integer ranking) {
    this.id = new RequestExpertId(request.getId(), expert.getId());
    this.request = request;
    this.expert = expert;
    this.ranking = ranking;
    updateStatus(RequestExpertStatus.QUEUED);
  }

  public void updateStatus(RequestExpertStatus status) {
    this.status = status;
    this.updatedOn = LocalDateTime.now();
    this.refusalReason = null;
    this.refusalJustification = null;
  }

  public void updateStatusRefusal(RefusalReason reason, String justification) {
    updateStatus(RequestExpertStatus.REFUSED);
    this.refusalReason = reason;
    this.refusalJustification = justification;
  }
}
