package fr.gouv.beta.selexpert.model;

import jakarta.persistence.*;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Collection;
import java.util.Collections;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.experimental.SuperBuilder;
import org.hibernate.annotations.ColumnDefault;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

@Entity
@Table(name = "_user", schema = "public")
@Inheritance(strategy = InheritanceType.JOINED)
@Data
@SuperBuilder
@AllArgsConstructor
@Audited(withModifiedFlag = true)
public class User implements UserDetails {

  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  private Integer id;

  @Column(nullable = false, unique = true)
  private String email;

  @Column(nullable = true)
  @NotAudited
  private String password;

  @Column(name = "name", nullable = false)
  @ColumnDefault("''")
  private String name;

  @Column(nullable = false)
  @Enumerated(EnumType.STRING)
  @NotAudited
  private Role role;

  @NotAudited @Column private LocalDateTime activatedOn;

  @Column(name = "self_registration")
  @NotAudited
  private Boolean selfRegistration;

  @Column(name = "all_notifications")
  private Boolean allNotifications;

  @Column(name = "last_active")
  @NotAudited
  private LocalDate lastActive;

  public User() {}

  @Override
  public Collection<? extends GrantedAuthority> getAuthorities() {
    return Collections.emptyList();
  }

  @Override
  public String getUsername() {
    return email;
  }

  @Override
  public boolean isAccountNonExpired() {
    return true;
  }

  @Override
  public boolean isAccountNonLocked() {
    return true;
  }

  @Override
  public boolean isCredentialsNonExpired() {
    return true;
  }

  @Override
  public boolean isEnabled() {
    return true;
  }
}
