package fr.gouv.beta.selexpert.repository;

import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;
import java.util.List;
import org.hibernate.envers.AuditReader;
import org.hibernate.envers.AuditReaderFactory;
import org.hibernate.envers.query.AuditEntity;
import org.hibernate.envers.query.criteria.MatchMode;
import org.springframework.stereotype.Repository;

@Repository
public class AuditRepository {

  @PersistenceContext private EntityManager entityManager;

  private AuditReader auditReader() {
    return AuditReaderFactory.get(entityManager);
  }

  public <T> List<Integer> getRevisionNumbers(Class<T> clazz) {
    return auditReader()
        .createQuery()
        .forRevisionsOfEntityWithChanges(clazz, true)
        .addProjection(AuditEntity.revisionNumber().distinct())
        .addOrder(AuditEntity.revisionNumber().desc())
        .getResultList();
  }

  public <T> List<T> getRevision(Integer revisionNumber, Class<T> clazz) {
    return auditReader()
        .createQuery()
        .forEntitiesModifiedAtRevision(clazz, revisionNumber)
        .getResultList();
  }

  public <T> T getRevisionFor(Object primaryKey, Integer revisionNumber, Class<T> clazz) {
    return auditReader().find(clazz, primaryKey, revisionNumber);
  }

  public <T> List getRevisionsFor(Object primaryKey, Class<T> clazz) {
    return auditReader()
        .createQuery()
        .forRevisionsOfEntityWithChanges(clazz, false)
        .add(AuditEntity.id().eq(primaryKey))
        .getResultList();
  }

  public <T> Object getEntityWithCurrentOrPreviousValue(
      String property, String value, Class<T> clazz) {
    return auditReader()
        .createQuery()
        .forRevisionsOfEntityWithChanges(clazz, false)
        .add(AuditEntity.property(property).ilike(value, MatchMode.EXACT))
        .addOrder(AuditEntity.revisionNumber().desc())
        .setMaxResults(1)
        .getSingleResult();
  }
}
