package fr.gouv.beta.selexpert.repository;

import fr.gouv.beta.selexpert.dto.ExpertFilters;
import fr.gouv.beta.selexpert.model.Expert;
import jakarta.persistence.criteria.*;
import java.text.MessageFormat;
import java.util.List;
import java.util.Objects;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.jpa.domain.Specification;

public class ExpertSpecifications {

  static Specification<Expert> isRegisteredAt(List<Integer> courtIds) {
    return (root, query, builder) ->
        CollectionUtils.isNotEmpty(courtIds)
            ? root.get("appealCourt").get("id").in(courtIds)
            : null;
  }

  static Specification<Expert> hasExpertise(String expertise) {
    return (root, query, builder) -> {
      query.distinct(
          true); // each matched expertise will return one row (if general expertise like A-)
      return Objects.nonNull(expertise)
          ? startsWith(
              expertise, builder, root.get("expertExpertisesNew").get("expertise").get("code"))
          : null;
    };
  }

  static Specification<Expert> nameStartsWith(String nameChunk) {
    return (root, query, builder) ->
        Objects.nonNull(nameChunk) ? fullTextSearch(nameChunk, builder, root.get("name")) : null;
  }

  static Specification<Expert> includeDeleted(Boolean includeDeleted) {
    return (root, query, builder) ->
        Objects.isNull(includeDeleted) || !includeDeleted
            ? builder.or(builder.isFalse(root.get("deleted")), builder.isNull(root.get("deleted")))
            : null;
  }

  public static Specification<Expert> all(ExpertFilters expertFilters) {
    return Specification.allOf(
        isRegisteredAt(expertFilters.getCourtIds()),
        hasExpertise(expertFilters.getExpertise()),
        nameStartsWith(expertFilters.getNameChunk()),
        includeDeleted(expertFilters.getIncludeDeleted()));
  }

  private static Predicate startsWith(
      String text, CriteriaBuilder builder, Expression<String> expression) {
    return builder.like(expression, MessageFormat.format("{0}%", text));
  }

  private static Predicate fullTextSearch(
      String text, CriteriaBuilder builder, Expression<String> expression) {
    return builder.like(
        builder.function("unaccent", String.class, builder.lower(expression)),
        MessageFormat.format("{0}%", StringUtils.stripAccents(text.toLowerCase())));
  }
}
