package fr.gouv.beta.selexpert.repository;

import fr.gouv.beta.selexpert.model.Expert;
import fr.gouv.beta.selexpert.model.Jurisdiction;
import fr.gouv.beta.selexpert.model.Request;
import fr.gouv.beta.selexpert.model.RequestStatus;
import fr.gouv.beta.selexpert.model.UserClerk;
import fr.gouv.beta.selexpert.model.UserMagistrate;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Query;

public interface RequestRepository
    extends JpaRepository<Request, Integer>, JpaSpecificationExecutor<Request> {

  Optional<Request> findByIdAndJurisdictionAndMagistrate(
      Integer requestId, Jurisdiction jurisdiction, UserMagistrate magistrate);

  Optional<Request> findByIdAndJurisdictionAndRequestClerks(
      Integer requestId, Jurisdiction jurisdiction, UserClerk clerk);

  Page<Request> findByRequestExpertsExpertAndRequestExpertsSentOnNotNull(
      Expert expert, Pageable pageable);

  List<Request> findByStatusAndDeliberationDateBefore(RequestStatus open, LocalDate today);

  int countByMagistrateAndClerkIsNull(UserMagistrate magistrate);

  int countByClerk(UserClerk clerk);

  @Query(
      value =
          "SELECT r FROM Request r JOIN RequestExpert re ON r = re.request WHERE NOT r.archived GROUP BY r HAVING MAX(re.updatedOn) < :before")
  List<Request> getRequestsToBeArchived(LocalDateTime before);

  @Query(
      value =
          "select floor(extract(epoch FROM avg(re.updated_on - r.created_on)) / 3600) from request_expert re join request r on r.id = re.request_id where not r.is_urgent and re.status = 'ACCEPTED'",
      nativeQuery = true)
  Integer getAvgAcceptationTime();

  @Query(
      value =
          "SELECT * FROM Request r WHERE r.deliberation_date - :today = :daysToRemindDeliberation AND r.status = 'OPEN'",
      nativeQuery = true)
  List<Request> getRequestsToRemindDeliberation(LocalDate today, Integer daysToRemindDeliberation);

  @Query(
      value =
          "SELECT ((page-1) / :pageSize) FROM (SELECT id, COUNT(*) OVER (ORDER BY r.created_on DESC) page FROM request r WHERE r.magistrate_id = :magistrateId) x WHERE id = :requestId",
      nativeQuery = true)
  Integer getRequestPageNumberForMagistrate(
      Integer requestId, Integer magistrateId, Integer pageSize);

  @Query(
      value =
          "SELECT ((page-1) / :pageSize) FROM (SELECT id, COUNT(*) OVER (ORDER BY r.created_on DESC) page FROM request r JOIN request_clerk rc ON r.id = rc.request_id WHERE rc.clerk_id = :clerkId) x WHERE id = :requestId",
      nativeQuery = true)
  Integer getRequestPageNumberForClerk(Integer requestId, Integer clerkId, Integer pageSize);

  @Query(
      value =
          "SELECT ((page-1) / :pageSize) FROM (SELECT id, COUNT(*) OVER (ORDER BY r.created_on DESC) page FROM request r JOIN request_expert re ON r.id = re.request_id WHERE re.expert_id = :expertId) x WHERE id = :requestId",
      nativeQuery = true)
  Integer getRequestPageNumberForExpert(Integer requestId, Integer expertId, Integer pageSize);

  @Query(
      value =
          "SELECT ((page-1) / :pageSize) FROM (SELECT id, COUNT(*) OVER (ORDER BY r.created_on DESC) page FROM request r) x WHERE id = :requestId",
      nativeQuery = true)
  Integer getRequestPageNumberForAdmin(Integer requestId, Integer pageSize);
}
