package fr.gouv.beta.selexpert.repository;

import fr.gouv.beta.selexpert.dto.RequestFilters;
import fr.gouv.beta.selexpert.model.Request;
import java.text.MessageFormat;
import java.util.Objects;
import org.springframework.data.jpa.domain.Specification;

public class RequestSpecifications {

  static Specification<Request> atAppealCourt(Integer appealCourtId) {
    return (root, query, builder) ->
        Objects.nonNull(appealCourtId)
            ? builder.equal(root.get("jurisdiction").get("appealCourt").get("id"), appealCourtId)
            : null;
  }

  static Specification<Request> atJurisdiction(Integer jurisdictionId) {
    return (root, query, builder) ->
        Objects.nonNull(jurisdictionId)
            ? builder.equal(root.get("jurisdiction").get("id"), jurisdictionId)
            : null;
  }

  static Specification<Request> magistrateIs(Integer magistrateId) {
    return (root, query, builder) ->
        Objects.nonNull(magistrateId)
            ? builder.equal(root.get("magistrate").get("id"), magistrateId)
            : null;
  }

  static Specification<Request> clerkIs(Integer clerkId) {
    return (root, query, builder) ->
        Objects.nonNull(clerkId)
            ? builder.equal(root.get("requestClerks").get("id"), clerkId)
            : null;
  }

  static Specification<Request> expertIs(Integer expertId) {
    return (root, query, builder) ->
        Objects.nonNull(expertId)
            ? builder.equal(root.get("requestExperts").get("expert").get("id"), expertId)
            : null;
  }

  static Specification<Request> procNumStartsWith(String procNum) {
    return (root, query, builder) ->
        Objects.nonNull(procNum)
            ? builder.like(root.get("procedureId"), MessageFormat.format("{0}%", procNum))
            : null;
  }

  static Specification<Request> creatorMagistrateIs(Integer magistrateId) {
    return (root, query, builder) ->
        Objects.nonNull(magistrateId)
            ? builder.and(
                builder.equal(root.get("magistrate").get("id"), magistrateId),
                builder.isNull(root.get("clerk").get("id")))
            : null;
  }

  static Specification<Request> creatorClerkIs(Integer clerkId) {
    return (root, query, builder) ->
        Objects.nonNull(clerkId) ? builder.equal(root.get("clerk").get("id"), clerkId) : null;
  }

  public static Specification<Request> all(RequestFilters requestFilters) {
    return Specification.allOf(
        atAppealCourt(requestFilters.getAppealCourtId()),
        atJurisdiction(requestFilters.getJurisdictionId()),
        magistrateIs(requestFilters.getMagistrateId()),
        clerkIs(requestFilters.getClerkId()),
        expertIs(requestFilters.getExpertId()),
        procNumStartsWith(requestFilters.getProcNumChunk()),
        creatorMagistrateIs(requestFilters.getCreatorMagistrateId()),
        creatorClerkIs(requestFilters.getCreatorClerkId()));
  }
}
