package fr.gouv.beta.selexpert.repository;

import fr.gouv.beta.selexpert.model.Role;
import fr.gouv.beta.selexpert.model.User;
import java.util.List;
import java.util.Optional;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;

public interface UserRepository extends JpaRepository<User, Integer> {
  Optional<User> findByEmail(String email);

  boolean existsByEmail(String email);

  @Query(
      value =
          """
          select u.id from _user u
          left join _user_magistrate m on u.id = m.id
          left join jurisdiction jm on m.jurisdiction_id = jm.id
          left join _user_clerk c on u.id = c.id
          left join jurisdiction jc on c.jurisdiction_id = jc.id
          where role in ('MAGISTRATE','CLERK')
          order by u.id desc
          """,
      nativeQuery = true)
  Page<Integer> findCourtUserIds(Pageable pageable);

  @Query(
      value =
          """
          select u.id from _user u
          left join _user_magistrate m on u.id = m.id
          left join jurisdiction jm on m.jurisdiction_id = jm.id
          left join _user_clerk c on u.id = c.id
          left join jurisdiction jc on c.jurisdiction_id = jc.id
          where role in ('MAGISTRATE','CLERK')
          and jm.id = :jurisdictionId or jc.id = :jurisdictionId
          order by u.id desc
          """,
      nativeQuery = true)
  Page<Integer> findCourtUserIdsOfJurisdiction(Integer jurisdictionId, Pageable pageable);

  @Query(
      value =
          """
          select u.id from _user u
          left join _user_magistrate m on u.id = m.id
          left join jurisdiction jm on m.jurisdiction_id = jm.id
          left join _user_clerk c on u.id = c.id
          left join jurisdiction jc on c.jurisdiction_id = jc.id
          where role in ('MAGISTRATE','CLERK')
          and jm.appeal_court_id = :appealCourtId or jc.appeal_court_id = :appealCourtId
          order by u.id desc
          """,
      nativeQuery = true)
  Page<Integer> findCourtUserIdsOfAppealCourt(Integer appealCourtId, Pageable pageable);

  List<User> findAllByIdInOrderByIdDesc(List<Integer> ids);

  List<User> findAllByRoleInOrderByName(Role... role);
}
