package fr.gouv.beta.selexpert.service;

import fr.gouv.beta.selexpert.AppException;
import fr.gouv.beta.selexpert.dto.ExpertCreationForm;
import fr.gouv.beta.selexpert.model.*;
import fr.gouv.beta.selexpert.repository.AppealCourtRepository;
import fr.gouv.beta.selexpert.repository.ExpertRepository;
import fr.gouv.beta.selexpert.repository.ExpertiseNewRepository;
import fr.gouv.beta.selexpert.repository.UserRepository;
import fr.gouv.beta.selexpert.util.TextUtils;
import jakarta.transaction.Transactional;
import java.util.HashSet;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

@Service
@RequiredArgsConstructor
public class AdminService {

  private final ExpertRepository expertRepository;
  private final UserRepository userRepository;
  private final ExpertiseNewRepository expertiseNewRepository;
  private final AppealCourtRepository appealCourtRepository;
  private final AuthService authService;

  @Transactional
  public void createExpert(ExpertCreationForm expertCreationForm) {

    if (expertRepository.findByEmail(expertCreationForm.getEmail()).isPresent()
        || userRepository.findByEmail(expertCreationForm.getEmail()).isPresent()) {
      throw new AppException(expertCreationForm.getEmail() + " already registered");
    }

    Expert expert = new Expert();
    expert.setManuallyAdded(true);
    expert.setAppealCourt(
        appealCourtRepository.findById(expertCreationForm.getCourtId()).orElseThrow());
    if (!"Entreprise".equals(expertCreationForm.getTitle())) {
      expert.setTitle(expertCreationForm.getTitle());
      expert.setName(
          expertCreationForm.getLastName().toUpperCase() + " " + expertCreationForm.getFirstName());
    } else {
      expert.setName(expertCreationForm.getLastName().toUpperCase());
    }
    expert.setEmail(expertCreationForm.getEmail());
    expert.setHonorary(expertCreationForm.isHonorary());
    expert.setNotRegistered(expertCreationForm.isNotRegistered());
    if (StringUtils.isNotBlank(expertCreationForm.getAddress()))
      expert.setAddress(TextUtils.cleanValue(expertCreationForm.getAddress()));
    expert.setZipCode(StringUtils.defaultIfBlank(expertCreationForm.getZipCode(), null));
    if (StringUtils.isNotBlank(expertCreationForm.getCity()))
      expert.setCity(expertCreationForm.getCity().toUpperCase());
    expert.setPhoneMobile(expertCreationForm.getPhoneMobile());
    expert.setPhoneLandline(expertCreationForm.getPhoneLandline());
    expert.setExpertExpertisesNew(new HashSet<>());
    expertCreationForm.getExpertiseIds().stream()
        .distinct()
        .forEach(
            expertiseId -> {
              ExpertiseNew expertise = expertiseNewRepository.findById(expertiseId).orElseThrow();
              expert.getExpertExpertisesNew().add(new ExpertExpertiseNew(expert, expertise, null));
            });
    expertRepository.save(expert);
    if (expertCreationForm.isSendActivationEmail())
      authService.activateExpert(expertCreationForm.getEmail());
  }

  @Transactional
  public void deleteExpert(Integer expertId) {
    expertRepository.findById(expertId).ifPresent(expert -> expert.setDeleted(true));
  }
}
