package fr.gouv.beta.selexpert.service;

import jakarta.annotation.PostConstruct;
import java.util.List;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import sendinblue.ApiClient;
import sendinblue.ApiException;
import sendinblue.Configuration;
import sendinblue.auth.ApiKeyAuth;
import sibApi.TransactionalEmailsApi;
import sibModel.SendSmtpEmail;
import sibModel.SendSmtpEmailSender;
import sibModel.SendSmtpEmailTo;

@Service
@RequiredArgsConstructor
@Slf4j
public class MailSenderService {

  @Value("${selexpert.mail.api-key}")
  private String mailApiKey;

  @Value("${selexpert.mail.sender-name}")
  private String senderName;

  @Value("${selexpert.mail.sender-email}")
  private String senderEmail;

  private SendSmtpEmailSender sender;

  @PostConstruct
  public void init() {
    ApiClient defaultClient = Configuration.getDefaultApiClient();
    ApiKeyAuth apiKey = (ApiKeyAuth) defaultClient.getAuthentication("api-key");
    apiKey.setApiKey(mailApiKey);
    sender = new SendSmtpEmailSender().name(senderName).email(senderEmail);
  }

  public void send(String recipient, String subject, String content) throws ApiException {
    SendSmtpEmail sendSmtpEmail =
        new SendSmtpEmail()
            .sender(sender)
            .to(List.of(new SendSmtpEmailTo().email(recipient)))
            .subject(subject)
            .htmlContent(content);
    new TransactionalEmailsApi().sendTransacEmail(sendSmtpEmail); // TODO thread safe?
  }
}
