package fr.gouv.beta.selexpert.service;

import fr.gouv.beta.selexpert.model.Expert;
import fr.gouv.beta.selexpert.model.Role;
import fr.gouv.beta.selexpert.model.User;
import fr.gouv.beta.selexpert.model.UserExpert;
import fr.gouv.beta.selexpert.repository.UserRepository;
import jakarta.transaction.Transactional;
import java.time.LocalDateTime;
import java.util.Objects;
import lombok.RequiredArgsConstructor;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

@Service
@RequiredArgsConstructor
public class UserService {

  private final UserRepository userRepository;
  private final PasswordEncoder passwordEncoder;

  @Transactional
  public void createExpertUser(Expert expert, String password) {
    var user =
        UserExpert.builder()
            .role(Role.EXPERT)
            .email(expert.getEmail())
            .password(passwordEncoder.encode(password))
            .expert(expert)
            .activatedOn(LocalDateTime.now());
    user.name(expert.getName());
    userRepository.save(user.build());
  }

  @Transactional
  public void updateUserPassword(User user, String password) {
    user.setPassword(passwordEncoder.encode(password));
    if (Objects.isNull(user.getActivatedOn())) {
      user.setActivatedOn(LocalDateTime.now());
    }
  }
}
