package fr.gouv.beta.selexpert.util;

import java.time.DayOfWeek;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.ZoneOffset;
import java.util.List;

public class DateUtils {

  public static LocalDateTime UTCtoFR(LocalDateTime utcLocalDateTime) {
    return utcLocalDateTime
        .atZone(ZoneOffset.UTC)
        .withZoneSameInstant(ZoneId.of("Europe/Paris"))
        .toLocalDateTime();
  }

  /*
  input is UTC, output is FR timezone

  keep in mind when doing calculations related to the deadline, it must be compared to another
  FR timezone date
  */
  public static LocalDateTime getDeadline(LocalDateTime date, int weekdaysToReply) {
    int weekdaysAdded = 0;
    LocalDateTime deadline = UTCtoFR(date).toLocalDate().atTime(LocalTime.MAX);
    while (weekdaysAdded < weekdaysToReply) {
      deadline = deadline.plusDays(1);
      if (isWeekday(deadline)) {
        weekdaysAdded++;
      }
    }
    return deadline;
  }

  private static boolean isWeekday(LocalDateTime day) {
    return !List.of(DayOfWeek.SATURDAY, DayOfWeek.SUNDAY).contains(day.getDayOfWeek());
  }
}
