package fr.gouv.beta.selexpert.util;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;
import java.io.IOException;
import java.time.LocalDateTime;

public class LocalDateTimeSerializer extends StdSerializer<LocalDateTime> {

  public LocalDateTimeSerializer() {
    super(LocalDateTime.class);
  }

  /*
  All dates LocalDateTime are stored as UTC so for display purpose, calculate FR time.
  LocalDateTime has no TZ so it will be serialized as 2023-11-26T09:00:00

  In Angular, the date pipe will take this STRING (not a JS Date object) as param and display as is
  because it doesn't have a TZ.

  If the string had a TZ (or ended with Z), the date pipe would automatically adapt to the local
  system time which is not ideal...

  The deadline is transformed to a LocalDate (FR timezone) and the 23:59 part is added in the UI.
   */

  @Override
  public void serialize(LocalDateTime value, JsonGenerator gen, SerializerProvider provider)
      throws IOException {
    gen.writeString(DateUtils.UTCtoFR(value).toString());
  }
}
