package fr.gouv.beta.selexpert.dto;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

import jakarta.validation.ConstraintViolation;
import jakarta.validation.Validation;
import jakarta.validation.Validator;
import jakarta.validation.ValidatorFactory;
import java.util.Set;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;

public class PasswordFormTest {

  private static Validator validator;

  @BeforeAll
  public static void setUp() {
    ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
    validator = factory.getValidator();
  }

  @ParameterizedTest
  @ValueSource(
      strings = {
        "abc123DEF!",
        "abc123DEF!?.:,;@#$%^&*-=+_'\"~`|()[]{}<>/\\",
        "abc123DEF!abc123DEF!abc123DEF!abc123DEF!abc123DEF!abc123DEF!abc123DEF!abc123DEF!abc123DEF!abc123DEF!"
      })
  public void password_valid(String password) {
    assertTrue((isValid(password)));
  }

  @ParameterizedTest
  @ValueSource(
      strings = {
        "",
        "abc123 DEF!", // space not allowed
        "äbc123DEF!", // character not allowed
        "ab12DE!@#", // too short, 9 characters
        "ABC123DEF!", // no lowercase
        "abc123def!", // no uppercase
        "abcd123DEF", // no symbol
        "abc123DEF!abc123DEF!abc123DEF!abc123DEF!abc123DEF!abc123DEF!abc123DEF!abc123DEF!abc123DEF!abc123DEF!@" // too long, 101 characters
      })
  public void password_invalid(String password) {
    assertFalse(isValid(password));
  }

  private boolean isValid(String password) {
    PasswordForm passwordForm = new PasswordForm();
    passwordForm.setPassword(password);
    Set<ConstraintViolation<PasswordForm>> violations = validator.validate(passwordForm);
    return violations.isEmpty();
  }
}
