package fr.gouv.beta.selexpert.service;

import static org.assertj.core.api.BDDAssertions.then;
import static org.mockito.BDDMockito.given;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

import fr.gouv.beta.selexpert.dto.UserInfo;
import fr.gouv.beta.selexpert.model.AuthActivity;
import fr.gouv.beta.selexpert.model.Role;
import fr.gouv.beta.selexpert.model.UserMagistrate;
import fr.gouv.beta.selexpert.repository.AuthActivityRepository;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Optional;
import org.junit.jupiter.api.extension.ExtendWith;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

@ExtendWith(MockitoExtension.class)
class AuthServiceTest {

  @Mock private AuthActivityRepository authActivityRepository;

  @InjectMocks private AuthService authService;

  @ParameterizedTest
  @ValueSource(strings = {"true", "false"})
  public void getUserInfo_activity(String flagStr) {

    // GIVEN
    var flag = Boolean.parseBoolean(flagStr);
    var now = LocalDate.now(ZoneId.of("Europe/Paris"));
    AuthActivity authActivity = AuthActivity.builder().count(6).build();
    UserMagistrate user =
        UserMagistrate.builder()
            .lastActive(flag ? now.minusDays(1) : null)
            .role(Role.MAGISTRATE)
            .build();
    given(authActivityRepository.findById(new AuthActivity.AuthActivityId(user.getRole(), now)))
        .willReturn(flag ? Optional.of(authActivity) : Optional.empty());

    // WHEN
    UserInfo userInfo = authService.getUserInfo(user);

    // THEN
    then(userInfo).isNotNull();
    then(user.getLastActive()).isEqualTo(now);
    verify(authActivityRepository, times(flag ? 0 : 1))
        .save(new AuthActivity(user.getRole(), now, 1));
    if (flag) then(authActivity.getCount()).isEqualTo(7);
  }
}
